import { useLayoutEffect, useState } from "react"
import React from "react"

const log = console.log
let cache: Array<any> = []
const clear = () => cache = []

const react18 = parseFloat(React.version) > 17
const referenceSet = new Set<any>()

export default <T>(factory: () => T, cleanup?: (val: T) => void): T => {
    if (react18) {
        const [value] = useState(factory)

        useLayoutEffect(() => {
            referenceSet.add(value)
            setTimeout(() => referenceSet.delete(value))

            return () => {
                if (referenceSet.has(value)) return
                cleanup?.(value)
            }
        }, [])

        queueMicrotask(() => {
            if (referenceSet.has(value)) return
            cleanup?.(value)
        })

        return value
    }

    const [result] = useState<T>(() => {
        if (log === console.log) {
            queueMicrotask(clear)
            const result = factory()
            cache.push(result)
            return result
        }
        return cache.shift()
    })
    
    useLayoutEffect(() => {
        return () => {
            cleanup?.(result)
        }
    }, [])

    return result
}